/* Classic Ladder Project */
/* Copyright (C) 2001 Marc Le Douarain */
/* mavati@club-internet.fr */
/* http://www.multimania.com/mavati/classicladder */
/* February 2001 */
/* Last update : 17 April. 2001 */
/* ---------------------------------------------- */
/* Load Rungs , Timers and Monostables parameters */
/* ---------------------------------------------- */
/* This library is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU Lesser General Public */
/* License as published by the Free Software Foundation; either */
/* version 2.1 of the License, or (at your option) any later version. */

/* This library is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU */
/* Lesser General Public License for more details. */

/* You should have received a copy of the GNU Lesser General Public */
/* License along with this library; if not, write to the Free Software */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

char ConvRawLineOfElements(char * RawLine,char y,StrRung * StorageRung)
{
    char * StartOfValue;
    char * EndOfValue;
    char x = 0;

    char EndOfLine;

    StartOfValue = RawLine;
    EndOfValue = RawLine;

    do
    {
        /* Extract Element Type */
        StartOfValue = EndOfValue;
        do
        {
            EndOfValue++;
        }
        while(*EndOfValue!='-');
        *EndOfValue++ = '\0';
        StorageRung->Element[x][y].Type = atoi(StartOfValue);

        /* Extract ConnectedWithTop */
        StartOfValue = EndOfValue;
        do
        {
            EndOfValue++;
        }
        while(*EndOfValue!='-');
        *EndOfValue++ = '\0';
        StorageRung->Element[x][y].ConnectedWithTop = atoi(StartOfValue);

        /* Extract Var Type */
        StartOfValue = EndOfValue;
        do
        {
            EndOfValue++;
        }
        while(*EndOfValue!='/');
        *EndOfValue++ = '\0';
        StorageRung->Element[x][y].VarType = atoi(StartOfValue);

        /* Extract Var Offset in the type table */
        StartOfValue = EndOfValue;
        do
        {
            EndOfValue++;
        }
        while( (*EndOfValue!=',') && (*EndOfValue!=10) );
        EndOfLine = TRUE;
        if (*EndOfValue==',')
            EndOfLine = FALSE;
        *EndOfValue++ = '\0';
        StorageRung->Element[x][y].VarNum = atoi(StartOfValue);

        /* Next Element */
        x++;

    }
    while(!EndOfLine);
    return (x);
}

char LoadRung(char * FileName,StrRung * BufRung)
{
    FILE * File;
    char Okay = FALSE;
    char Line[300];
    char * LineOk;
    char y = 0;
    File = fopen(FileName,"rt");
    if (File)
    {
        do
        {
            LineOk = fgets(Line,300,File);
            if (LineOk)
            {
                char Element;
                char FndElements;
                switch(Line[0])
                {
                    case ';':
                        break;
                    case '#':
                        if(strncmp(&Line[1],"VER=",4)==0)
                        {
                            if (atoi(&Line[5])>1)
                            {
                                printf("Rung version not supported...\n");
                                LineOk = FALSE;
                            }
                        }
                        if(strncmp(&Line[1],"LABEL=",6)==0)
                            strcpy(BufRung->Label,&Line[7]);
                        if(strncmp(&Line[1],"COMMENT=",8)==0)
                            strcpy(BufRung->Comment,&Line[9]);
                        break;
                    default:
                    FndElements = ConvRawLineOfElements(Line,y,BufRung);
                    y++;
                }
            }
        }
        while(LineOk);
        fclose(File);
        Okay = TRUE;
    }
    return (Okay);
}

void LoadAllRungs(char * BaseName,StrRung * Rungs,int * TheFirst,int * TheLast,int * TheCurrent)
{
    int NumRung;
    StrRung * PrevRung = NULL;
    int PrevNumRung = 0;
    *TheFirst = -1;
    *TheCurrent = -1;
    for(NumRung=0; NumRung<NBR_RUNGS; NumRung++)
    {
        char RungFile[25];
        sprintf(RungFile,"%s%d.csv",BaseName,NumRung);
        printf("Loading file : %s",RungFile);
        if (LoadRung(RungFile,Rungs))
        {
            if (*TheFirst==-1)
            {
                *TheFirst = NumRung;
                *TheCurrent = NumRung;
            }
            if (PrevRung)
            {
                PrevRung->NextRung = NumRung;
            }
            Rungs->Used = TRUE;
            Rungs->PrevRung = PrevNumRung;
            *TheLast = NumRung;
            PrevNumRung = NumRung;
            PrevRung = Rungs;
            printf(" - ok.\n");
        }
        else
            printf(" - not found.\n");
        Rungs++;
    }
}

void DumpRung(StrRung * TheRung)
{
    char x,y;
    for (y=0;y<RUNG_HEIGHT;y++)
    {
        for(x=0;x<RUNG_WIDTH;x++)
        {
            printf("%d:%d:%d=%d , ",TheRung->Element[x][y].Type,TheRung->Element[x][y].ConnectedWithTop,TheRung->Element[x][y].VarNum,TheRung->Element[x][y].DynamicOutput);
        }
        printf("\n");
    }
}

void ConvRawLineOfNumbers(char * RawLine,char NbrParams,int * ValuesFnd)
{
    char * StartOfValue;
    char * EndOfValue;
    char Num = 0;

    char EndOfLine;

    StartOfValue = RawLine;
    EndOfValue = RawLine;
    EndOfLine = FALSE;

    do
    {
        /* Extract Value */
        StartOfValue = EndOfValue;
        do
        {
            EndOfValue++;
        }
        while( (*EndOfValue!=',') && (*EndOfValue!=10) );
        if (*EndOfValue==10)
            EndOfLine = TRUE;
            *EndOfValue++ = '\0';
            *ValuesFnd++ = atoi(StartOfValue);
            Num++;
            StartOfValue = EndOfValue;
    }
    while( (!EndOfLine) && (Num<NbrParams) );
}

char LoadTimersParams(char * FileName,StrTimer * BufTimers)
{
    FILE * File;
    char Okay = FALSE;
    char Line[300];
    char * LineOk;
    int Params[3];
    File = fopen(FileName,"rt");
    if (File)
    {
        do
        {
            LineOk = fgets(Line,300,File);
            if (LineOk)
            {
                if (Line[0]!=';')
                {
                    ConvRawLineOfNumbers(Line,2,Params);
                    switch(Params[0])
                    {
                        case BASE_MINS:
                            BufTimers->Base = TIME_BASE_MINS;
                            BufTimers->Preset = Params[1] * TIME_BASE_MINS;
                            strcpy(BufTimers->DisplayFormat,"%.1fmn");
                            break;
                        case BASE_SECS:
                            BufTimers->Base = TIME_BASE_SECS;
                            BufTimers->Preset = Params[1] * TIME_BASE_SECS;
                            strcpy(BufTimers->DisplayFormat,"%.1fs");
                            break;
                        case BASE_100MS:
                            BufTimers->Base = TIME_BASE_100MS;
                            BufTimers->Preset = Params[1] * TIME_BASE_100MS;
                            strcpy(BufTimers->DisplayFormat,"%.0f00ms");
                            break;
                        default:
                            BufTimers->Base = 1;
                            BufTimers->Preset = 10;
                            strcpy(BufTimers->DisplayFormat,"%f?");
                            printf("!!! Error loading parameter base in %s\n",FileName);
                            break;
                    }
printf("Timer => Base = %d , Preset = %d\n",BufTimers->Base,BufTimers->Preset);
                    BufTimers++;
                }
            }
        }
        while(LineOk);
        fclose(File);
        Okay = TRUE;
    }
    return (Okay);
}

char LoadMonostablesParams(char * FileName,StrMonostable * BufMonostables)
{
    FILE * File;
    char Okay = FALSE;
    char Line[300];
    char * LineOk;
    int Params[3];
    File = fopen(FileName,"rt");
    if (File)
    {
        do
        {
            LineOk = fgets(Line,300,File);
            if (LineOk)
            {
                if (Line[0]!=';')
                {
                    ConvRawLineOfNumbers(Line,2,Params);
                    switch(Params[0])
                    {
                        case BASE_MINS:
                            BufMonostables->Base = TIME_BASE_MINS;
                            BufMonostables->Preset = Params[1] * TIME_BASE_MINS;
                            strcpy(BufMonostables->DisplayFormat,"%.1fmn");
                            break;
                        case BASE_SECS:
                            BufMonostables->Base = TIME_BASE_SECS;
                            BufMonostables->Preset = Params[1] * TIME_BASE_SECS;
                            strcpy(BufMonostables->DisplayFormat,"%.1fs");
                            break;
                        case BASE_100MS:
                            BufMonostables->Base = TIME_BASE_100MS;
                            BufMonostables->Preset = Params[1] * TIME_BASE_100MS;
                            strcpy(BufMonostables->DisplayFormat,"%fms");
                            break;
                        default:
                            BufMonostables->Base = 1;
                            BufMonostables->Preset = 10;
                            strcpy(BufMonostables->DisplayFormat,"%f?");
                            printf("!!! Error loading parameter base in %s\n",FileName);
                            break;
                    }
printf("Monostable => Base = %d , Preset = %d\n",BufMonostables->Base,BufMonostables->Preset);
                    BufMonostables++;
                }
            }
        }
        while(LineOk);
        fclose(File);
        Okay = TRUE;
    }
    return (Okay);
}
